var common = require('./common');

var grouperTests = {
    'basic grouper': function(done) {
        var grouper = new Grouper();
    
        // handle callbacks
        grouper.on('end', function(vals) {
            assert.equal(_(vals).compact().length, 3);
            done();
        });
        grouper.on('error', function(err) {
            throw err;
        });
    
        // setup group functions
        grouper.add(function(fn) {
            setTimeout(function() {
                fn(null, 1);
            }, (waitSeconds * 1000 * 3));
        });
        grouper.add(function(fn) {
            setTimeout(function() {
                fn(null, 2);
            }, (waitSeconds * 1000 * 2));
        });
        grouper.add(function(fn) {
            setTimeout(function() {
                fn(null, 3);
            }, (waitSeconds * 1000));
        });
    
        // run steps
        grouper.walk();
    },
    'grouper error': function(done) {
        var grouper = new Grouper();
    
        // handle callbacks
        grouper.on('end', done);
        grouper.on('error', function(err) {
            assert.equal(err.message, 'Unfunny parallel error');
            done();
        });
    
        // setup group functions
        grouper.add(function(fn) {
            setTimeout(function() {
                fn(null, 1);
            }, (waitSeconds * 1000 * 3));
        });
        grouper.add(function(fn) {
            setTimeout(function() {
                try {
                    throw new Error('Unfunny parallel error');
                } catch(err) {
                    fn(err, null);
                    return;
                }
                fn(null, 2);                
            }, (waitSeconds * 1000 * 2));
        });
        grouper.add(function(fn) {
            setTimeout(function() {
                fn(null, 3);
            }, (waitSeconds * 1000));
        });
    
        // run steps
        grouper.walk();
    },
    'grouper chainability': function(done) {
        var grouper = new Grouper();
    
        // handle callbacks
        grouper.on('end', function(vals) {
            assert.equal(_(vals).compact().length, 3);
            done();
        });
        grouper.on('error', function(err) {
            throw err;
        });
    
        // setup group functions
        grouper.add(function(fn) {
            setTimeout(function() {
                fn(null, 1);
            }, (waitSeconds * 1000 * 3));
        }).add(function(fn) {
            setTimeout(function() {
                fn(null, 2);
            }, (waitSeconds * 1000 * 2));
        }).add(function(fn) {
            setTimeout(function() {
                fn(null, 3);
            }, (waitSeconds * 1000));
        });
    
        // run steps
        grouper.walk();
    },
};

module.exports = grouperTests;
