var common = require('./common');

var stepperTests = {
    'basic stepper': function(done) {
        var stepper = new Stepper();
    
        // handle callbacks
        stepper.on('end', function(val) {
            assert.equal(val, 4);
            done();
        });
        stepper.on('error', function(err) {
            throw err;
        });
    
        // add steps
        stepper.add(function(err, val) {
            return 1;
        });
        stepper.add(function(err, val) {
            return 2;
        });
        stepper.add(function(err, val) {
            var self = this;
            setTimeout(function() {
                self();
            }, (waitSeconds * 1000));
        });
        stepper.add(function(err, val) {
            return 4;
        });
    
        // run steps
        stepper.walk();
    },
    'stepper error': function(done) {
        var stepper = new Stepper();
    
        // handle callbacks
        stepper.on('end', done);
        stepper.on('error', function(err) {
            assert.equal(err.message, 'Unfunny serial error');
        });
    
        // add steps
        stepper.add(function(err, val) {
            return 1;
        });
        stepper.add(function(err, val) {
            var self = this;
            try {
                throw new Error('Unfunny serial error');
            } catch(err) {
                self(err, null);
                return;
            }
        });
        stepper.add(function(err, val) {
            if (err) throw err;
            var self = this;
            setTimeout(function() {
                self();
            }, (waitSeconds * 1000));
        });
        stepper.add(function(err, val) {
            if (err) throw err;
            return 4;
        });
    
        // run steps
        stepper.walk();
    },
    'stepper chainability': function(done) {
        var stepper = new Stepper();
    
        // handle callbacks
        stepper.on('end', function(val) {
            assert.equal(val, 4);
            done();
        });
        stepper.on('error', function(err) {
            throw err;
        });
    
        // add steps in a chain
        stepper.add(function(err, val) {
            return 1;
        }).add(function(err, val) {
            return 2;
        }).add(function(err, val) {
            var self = this;
            setTimeout(function() {
                self();
            }, (waitSeconds * 1000));
        }).add(function(err, val) {
            return 4;
        });
    
        // run steps
        stepper.walk();
    }
};

module.exports = stepperTests;
